# Copyright (c) 2004 Jean-Yves Lefort
#               2006 Rex Tsai
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 3. Neither the name of Jean-Yves Lefort nor the names of its contributors
#    may be used to endorse or promote products derived from this software
#    without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
# CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
# INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
# BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
# TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
# ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
# OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

import sys, re, ST, gobject, gtk
from ST import _

### constants #################################################################

class FIELD:
    TITLE, BANNER, URL, HOMEPAGE = range(4)

RADIO_HINET_NET_HOME	        = "http://radio.hinet.net/radio/"
RADIO_HINET_NET_MUSIC_P1        = "http://radio.hinet.net/radio/mainPageType1-1.html"
RADIO_HINET_NET_MUSIC_P2        = "http://radio.hinet.net/radio/mainPageType1-2.html"
RADIO_HINET_NET_NEWS_P1         = "http://radio.hinet.net/radio/mainPageType0-1.html"
RADIO_HINET_NET_ENGLISH_P1      = "http://radio.hinet.net/radio/mainPageType3-1.html"
RADIO_HINET_NET_ENTERTAINMENT_P1 = "http://radio.hinet.net/radio/mainPageType4-1.html"
RADIO_HINET_NET_ENTERTAINMENT_P2 = "http://radio.hinet.net/radio/mainPageType4-2.html"

re_title                = re.compile("<td valign=\"bottom\" class=\"11gray\"> <div align=\"center\">(.*?)</div></td>")
re_url			= re.compile(r"PopProgram.do\?id=([0-9]+.*?)")
re_img			= re.compile("<td><img src=\"logos/(.*?)/(.*?\.(jpg|JPG|gif|GIF|png|PNG))\" width=\"30\" height=\"30\"></td>")
re_mms			= re.compile("\"(http://grm.cdn.hinet.net/media/radio/play.asp\?mmsurl=mms://live.media.hinet.net/.*?)\"");
re_mmsurl		= re.compile("\"(mms://.*?/live.media.hinet.net/.*?)\"");

### helpers ###################################################################

class struct:
    def __init__ (self, **entries):
        self.__dict__.update(entries)

def parse_error (err):
    handler.notice(_("parse error: %s") % (err))

### transfer callbacks ########################################################

def tunein_line_cb (line, url):
    match = re_mms.search(line)
    if match is not None:
        asxurl = match.group(1)
        session = ST.TransferSession()
        session.get_by_line(asxurl,
                flags = ST.TRANSFER_UTF8 | ST.TRANSFER_PARSE_HTTP_CHARSET | ST.TRANSFER_PARSE_HTML_CHARSET,
                body_cb = asx_cb,
                body_args = (url,))

def asx_cb (line, url):
    match = re_mmsurl.search(line)
    if match is not None \
            and match:
        url.append(match.group(1))

def streams_line_cb (line, streams, info):
    if not hasattr(info, "aborted"):

        match = re_title.search(line)
        if match is not None:
            info.stream = HinetNetStream()
            info.stream.name = match.group(1)
            info.stream.fields[FIELD.TITLE] = match.group(1)
 
        url = None
        match = re_url.search(line)
        if match is not None:
            url = RADIO_HINET_NET_HOME + match.group()

        if url is not None:
            if info.stream is None:
                info.stream = HinetNetStream()
            info.stream.fields[FIELD.HOMEPAGE] = url
            info.stream.fields[FIELD.URL] = "" + RADIO_HINET_NET_HOME + "player/player.jsp?radio_id=" + match.group(1)

        match = re_img.search(line)
        if match is not None:
            info.stream.image_url = RADIO_HINET_NET_HOME + "/logos/" + match.group(1) + "/" + match.group(2)
            streams.append(info.stream)


### stream implementation #####################################################

class HinetNetStream (ST.Stream):
    image_url = None

### handler implementation ####################################################

class HinetNetHandler (ST.Handler):
    def __new__ (cls):
        self = ST.Handler.__new__(cls, "radio.hinet.net.py")

        self.label = "radio.hinet.net"
        self.home = RADIO_HINET_NET_HOME
        try:
            self.icon = gtk.gdk.pixbuf_new_from_file(ST.find_icon("hinet.png"))
        except:
            pass
        self.cached_images = {}

        category = ST.Category()
        category.name = "__main"
        category.label = _("Internet Radios")
        category.url_postfix = RADIO_HINET_NET_NEWS_P1
        self.stock_categories = category,


        self.config.register(("download-images",
                              bool,
                              None,
                              None,
                              1,
                              gobject.PARAM_READWRITE))

        self.add_field(ST.HandlerField(FIELD.TITLE,
                                       _("Title"),
                                       str,
                                       ST.HANDLER_FIELD_VISIBLE,
                                       _("The show title")))
        self.add_field(ST.HandlerField(FIELD.BANNER,
                                       _("Show"),
                                       gtk.gdk.Pixbuf,
                                       ST.HANDLER_FIELD_VISIBLE,
                                       _("The show banner")))
        self.add_field(ST.HandlerField(FIELD.HOMEPAGE,
                                       _("Schedule"),
                                       str,
                                       ST.HANDLER_FIELD_VISIBLE
                                       | ST.HANDLER_FIELD_START_HIDDEN,
                                       _("The show schedule URL")))
        self.add_field(ST.HandlerField(FIELD.URL,
                                       _("URL"),
                                       str,
                                       ST.HANDLER_FIELD_VISIBLE
                                       | ST.HANDLER_FIELD_START_HIDDEN,
                                       _("The show URL")))

        return self

    def reload (self, category):
        if not hasattr(self, "categories"):
            categories = []

            cate = ST.Category()
            cate.name = _("Music")
            cate.url_postfix = RADIO_HINET_NET_MUSIC_P1
            cate.label = _("Music")
            categories.append(cate)

            cate = ST.Category()
            cate.name = _("News")
            cate.url_postfix = RADIO_HINET_NET_NEWS_P1
            cate.label = _("News")
            categories.append(cate)

            cate = ST.Category()
            cate.name = _("English")
            cate.url_postfix = RADIO_HINET_NET_ENGLISH_P1
            cate.label = _("English")
            categories.append(cate)

            cate = ST.Category()
            cate.name = _("Entertainment")
            cate.url_postfix = RADIO_HINET_NET_ENTERTAINMENT_P1
            cate.label = _("Entertainment")
            categories.append(cate)

            self.categories = categories

        streams = []
        if category.url_postfix == RADIO_HINET_NET_MUSIC_P1 or category.url_postfix is None:
            # main category
            session = ST.TransferSession()
            session.get_by_line(RADIO_HINET_NET_MUSIC_P1,
                    flags = ST.TRANSFER_UTF8 | ST.TRANSFER_PARSE_HTTP_CHARSET | ST.TRANSFER_PARSE_HTML_CHARSET,
                    body_cb = streams_line_cb,
                    body_args = (streams, struct(stream = None)))

            session = ST.TransferSession()
            session.get_by_line(RADIO_HINET_NET_MUSIC_P2,
                    flags = ST.TRANSFER_UTF8 | ST.TRANSFER_PARSE_HTTP_CHARSET | ST.TRANSFER_PARSE_HTML_CHARSET,
                    body_cb = streams_line_cb,
                    body_args = (streams, struct(stream = None)))

        elif category.url_postfix == RADIO_HINET_NET_ENTERTAINMENT_P1:
            session = ST.TransferSession()
            session.get_by_line(RADIO_HINET_NET_ENTERTAINMENT_P1,
                    flags = ST.TRANSFER_UTF8 | ST.TRANSFER_PARSE_HTTP_CHARSET | ST.TRANSFER_PARSE_HTML_CHARSET,
                    body_cb = streams_line_cb,
                    body_args = (streams, struct(stream = None)))

            session = ST.TransferSession()
            session.get_by_line(RADIO_HINET_NET_ENTERTAINMENT_P2,
                    flags = ST.TRANSFER_UTF8 | ST.TRANSFER_PARSE_HTTP_CHARSET | ST.TRANSFER_PARSE_HTML_CHARSET,
                    body_cb = streams_line_cb,
                    body_args = (streams, struct(stream = None)))
        else:
            session = ST.TransferSession()
            session.get_by_line(category.url_postfix,
                    flags = ST.TRANSFER_UTF8 | ST.TRANSFER_PARSE_HTTP_CHARSET | ST.TRANSFER_PARSE_HTML_CHARSET,
                    body_cb = streams_line_cb,
                    body_args = (streams, struct(stream = None)))

        for stream in streams:
            if stream.image_url is not None:
                if not self.cached_images.has_key(stream.image_url) and handler.config["download-images"]:
                    try:
                        buffer = session.get_binary(stream.image_url)
                    except ST.AbortError:
                        break
                    except:
                        handler.notice(_("unable to download image %s: %s") % (stream.image_url, sys.exc_value))
                    else:
                        try:
                            loader = gtk.gdk.PixbufLoader()
                            loader.set_size(30,30)
                            loader.write(buffer)
                            self.cached_images[stream.image_url] = loader.get_pixbuf().scale_simple(30, 30, gtk.gdk.INTERP_NEAREST)
                            loader.close()
                        except:
                            handler.notice(_("unable to load image %s: %s") % (stream.image_url, sys.exc_value))
                if self.cached_images.has_key(stream.image_url):
                    stream.fields[FIELD.BANNER] = self.cached_images[stream.image_url]
                
        return (self.categories, streams)

    def stream_get_stock_field (self, stream, stock_field):
        if stock_field == ST.HANDLER_STOCK_FIELD_NAME:
            return stream.fields[FIELD.TITLE]
        elif stock_field == ST.HANDLER_STOCK_FIELD_HOMEPAGE:
            return stream.fields[FIELD.HOMEPAGE]

    def stream_browse (self, stream):
    	ST.action_run("view-web", stream.fields[FIELD.HOMEPAGE])

    def preferences_widget_new (self):
        check = gtk.CheckButton(_("_Download images"))
        check.set_active(self.config["download-images"])
        check.connect("toggled", self.download_images_toggled_h)
        ST.set_tooltip(check, _("If this option is enabled, the show banners will be downloaded."))
        check.show()

        return ST.HIGSection(_("Images"), check)

    def stream_tune_in (self, stream):
        url = []
        session = ST.TransferSession()
        session.get_by_line(stream.fields[FIELD.URL],
                            flags = ST.TRANSFER_UTF8 | ST.TRANSFER_PARSE_HTTP_CHARSET | ST.TRANSFER_PARSE_HTML_CHARSET,
                            body_cb = tunein_line_cb,
                            body_args = (url,))
        if url[0] is not None:
            ST.action_run("play-mms", url[0])

    def stream_record (self, stream):
        url = []
        session = ST.TransferSession()
        session.get_by_line(stream.fields[FIELD.URL],
                            flags = ST.TRANSFER_UTF8 | ST.TRANSFER_PARSE_HTTP_CHARSET | ST.TRANSFER_PARSE_HTML_CHARSET,
                            body_cb = tunein_line_cb,
                            body_args = (url,))
        if url[0] is not None:
            ST.action_run("record-mms", url[0])

    def stream_browse (self, stream):
    	ST.action_run("view-web", stream.fields[FIELD.HOMEPAGE])

    def download_images_toggled_h (self, check):
        self.config["download-images"] = check.get_active()

### initialization ############################################################

def init ():
    global handler

    if not ST.check_api_version(2, 0):
        raise RuntimeError, _("API version mismatch")
    
    ST.action_register("play-mms", _("Listen to a mms://"), "xine %q")
    ST.action_register("record-mms", _("Record a mms://"), "xterm -e mimms %q")

    handler = HinetNetHandler()
    ST.handlers_add(handler)

init()
